@@ -21,7 +21,6 @@ import cgi
 from http.client import BadStatusLine
 import logging
 import re
-import signal
 import sys
 import time
 import urllib.parse
@@ -117,7 +116,7 @@ class ConnectionPool:
     and recycle_connection() puts it back in.  To recycle a
     connection, call conn.close(recycle=True).
 
-    There are limits to both the overal pool and the per-key pool.
+    There are limits to both the overall pool and the per-key pool.
     """
 
     def __init__(self, log, max_pool=10, max_tasks=5):
@@ -245,14 +244,14 @@ class Connection:
         self.key = None
 
     def stale(self):
-        return self.reader is None or self.reader._eof
+        return self.reader is None or self.reader.at_eof()
 
     def fileno(self):
         writer = self.writer
         if writer is not None:
-            transport = writer._transport
+            transport = writer.transport
             if transport is not None:
-                sock = transport._sock
+                sock = transport.get_extra_info('socket')
                 if sock is not None:
                     return sock.fileno()
         return None
@@ -510,7 +509,6 @@ class Fetcher:
                 self.response = yield from self.request.get_response()
                 self.body = yield from self.response.read()
                 h_conn = self.response.get_header('connection').lower()
-                h_t_enc = self.response.get_header('transfer-encoding').lower()
                 if h_conn != 'close':
                     self.request.close(recycle=True)
                     self.request = None
@@ -601,6 +599,8 @@ class Fetcher:
                   size,
                   '%d/%d' % (len(self.new_urls or ()), len(self.urls or ())),
                   file=file)
+        elif self.response is None:
+            print(self.url, 'no response object')
         else:
             size = len(self.body or b'')
             if self.response.status == 200:
