@@ -248,20 +248,27 @@ def runtests():
                                 )
         cov.start()
 
-    finder = TestsFinder(args.testsdir, includes, excludes)
     logger = logging.getLogger()
     if v == 0:
-        logger.setLevel(logging.CRITICAL)
+        level = logging.CRITICAL
     elif v == 1:
-        logger.setLevel(logging.ERROR)
+        level = logging.ERROR
     elif v == 2:
-        logger.setLevel(logging.WARNING)
+        level = logging.WARNING
     elif v == 3:
-        logger.setLevel(logging.INFO)
+        level = logging.INFO
     elif v >= 4:
-        logger.setLevel(logging.DEBUG)
+        level = logging.DEBUG
+    logging.basicConfig(level=level)
+
+    finder = TestsFinder(args.testsdir, includes, excludes)
     if catchbreak:
         installHandler()
+    import asyncio.coroutines
+    if asyncio.coroutines._DEBUG:
+        print("Run tests in debug mode")
+    else:
+        print("Run tests in release mode")
     try:
         if args.forever:
             while True:
